@@ -1,10 +1,10 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: apachex.tcl,v 2.9 2001/02/18 18:18:38 jfontain Exp $}
+# $Id: apachex.tcl,v 2.16 2004/01/01 11:39:06 jfontain Exp $
 
 
-package provide apachex [lindex {$Revision: 2.9 $} 1]
+package provide apachex [lindex {$Revision: 2.16 $} 1]
 package require miscellaneous 1
 package require apacheutilities 1
 
@@ -42,7 +42,7 @@ namespace eval apachex {
     close $file
 
     proc initialize {optionsName} {
-        upvar $optionsName options
+        upvar 1 $optionsName options
         variable url
         variable data
         variable host
@@ -60,7 +60,7 @@ namespace eval apachex {
 
         if {[info exists transaction]} return                                                                                ;# busy
         if {[catch {set transaction [::http::geturl $url -command ::apachex::completed]} message]} {
-            flashMessage "apachex error: $message"
+            flashMessage "error: $message"
             cleanupRows voidArray
             incr data(updates)                                                                                   ;# show void values
         }
@@ -72,10 +72,10 @@ namespace eval apachex {
         variable transaction
         variable host
 
-        set valid [expr {[::http::ncode $token]==200}]                                        ;# sole HTTP code for good transaction
+        set valid [expr {[::http::ncode $token] == 200}]                                      ;# sole HTTP code for good transaction
         if {$valid} {
             set body [set ${token}(body)]                                ;# keep first table rows except the 1st row (column titles)
-            set lines [string range $body [expr {[string first </tr> $body]+5}] [expr {[string first </table> $body]-1}]]
+            set lines [string range $body [expr {[string first </tr> $body] + 5}] [expr {[string first </table> $body] - 1}]]
             regsub -all \n [string trim $lines] {} lines                                         ;# carriage returns must be ignored
             regsub -all \t $lines { } lines                         ;# replace all tabs by spaces as tabs will be used as separators
             regsub {^<tr[^>]*>} $lines {} lines                                                  ;# remove extreme table row markers
@@ -89,7 +89,7 @@ namespace eval apachex {
                 set column 0
                 foreach cell [split $line \t] {
                     regsub -all {<[^>]+>} $cell {} cell                                    ;# remove all remaining (formatting) tags
-                    if {$column==0} {
+                    if {$column == 0} {
                         set generation ?
                         scan $cell %u-%u row generation                  ;# generation is set here in apache version 1.3.6 and above
                         set current($row) {}                                        ;# keep track of current rows for latter cleanup
@@ -102,17 +102,17 @@ namespace eval apachex {
                 }
                 set data($row,6) [formattedTime $data($row,6)]
                 if {[info exists last($row,10)]} {
-                    set data($row,14) [expr {($data($row,10)-$last($row,10))*1000.0}]                ;# dynamic traffic in kilobytes
+                    set data($row,14) [expr {($data($row,10) - $last($row,10)) * 1000.0}]            ;# dynamic traffic in kilobytes
                 } else {
                     set data($row,14) ?
                 }
                 set last($row,10) $data($row,10)                         ;# save megabytes for slot for dynamic traffic calculations
             }
         } else {
-            if {[::http::ncode $token]==404} {
-                flashMessage "apachex error: no server status on $host"
+            if {[::http::ncode $token] == 404} {
+                flashMessage "error: no server status on $host"
             } else {
-                flashMessage "apachex error: [::http::code $token]"
+                flashMessage "error: [::http::code $token]"
             }
         }
         ::http::cleanup $token                                                                              ;# free transaction data
@@ -122,13 +122,13 @@ namespace eval apachex {
     }
 
     proc cleanupRows {currentNames} {
-        upvar $currentNames current
+        upvar 1 $currentNames current
         variable data
         variable last
 
         foreach {name row} [array get data *,0] {                                                      ;# cleanup vanished rows data
             if {[info exists current($row)]} continue
-            for {set column 0} {$column<=14} {incr column} {
+            for {set column 0} {$column <= 14} {incr column} {
                 unset data($row,$column)
             }
             catch {unset last($row,10)}                                                          ;# previous poll data may not exist
