@@ -1,23 +1,27 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: databar.tcl,v 2.15 2001/01/07 15:48:47 jfontain Exp $}
+# $Id: databar.tcl,v 2.31 2004/01/01 11:39:06 jfontain Exp $
 
 
 class dataBarChart {
 
+    set (plotBackground) $widget::option(button,background)
+
     proc dataBarChart {this parentPath args} composite {[new frame $parentPath] $args} blt2DViewer {
-        $widget::($this,path) [blt::barchart $widget::($this,path).bar\
-            -title {} -bottommargin 6 -topmargin 3 -bufferelements 0 -plotborderwidth 1\
-            -plotbackground $widget::option(button,background)\
-        ]
+        $widget::($this,path) [blt::barchart $widget::($this,path).bar -title {} -bottommargin 6 -plotbackground $(plotBackground)]
     } {
         set path $widget::($this,path).bar
         $path grid off                                                                                      ;# grid is on by default
         $path xaxis configure -hide 1
-        pack $path -side left -fill both -expand 1
+        $path yaxis configure -hide 1
+        # arrange for labels to remain priorly visible and the chart minimally so:
+        grid columnconfigure $widget::($this,path) 0 -minsize 70 -weight 1
+        grid $path -row 0 -column 0 -sticky nsew
         set ($this,path) $path
         composite::complete $this
+        # display help message after object is completely constructed for the message virtual procedure to work:
+        after idle "dataBarChart::updateMessage $this"
     }
 
     proc ~dataBarChart {this} {
@@ -29,14 +33,24 @@ class dataBarChart {
     proc options {this} {
         # force size values
         return [list\
+            [list -cellcolors {} {}]\
             [list -deletecommand {} {}]\
             [list -draggable 0 0]\
             [list -height $global::viewerHeight]\
             [list -mode normal normal]\
             [list -width $global::viewerWidth]\
+            [list -ymaximum {} {}]\
+            [list -ymaximumcell {} {}]\
         ]
     }
 
+    proc set-cellcolors {this value} {                             ;# colors of soon to be created cells when initializing from file
+        if {$composite::($this,complete)} {
+            error {option -cellcolors cannot be set dynamically}
+        }
+        blt2DViewer::setCellColors $this $value
+    }
+
     proc set-deletecommand {this value} {}
 
     foreach option {-height -width} {
@@ -56,12 +70,47 @@ class dataBarChart {
         $($this,path) configure -barmode $value
     }
 
+    proc set-ymaximum {this value} {
+        blt2DViewer::ySetMaximum $this $value
+    }
+    proc set-ymaximumcell {this value} {
+        blt2DViewer::ySetMaximumCell $this $value
+    }
+
     proc newElement {this path args} {                                                          ;# invoked from 2D viewer base class
         return [eval new element $path $args]
     }
 
-    proc updateElement {this element seconds value} {                           ;# value is either a valid number or the ? character
-        element::update $element $value
+    virtual proc updateElement {this element seconds value} {                   ;# value is either a valid number or the ? character
+        if {[string equal $value ?]} {                                                                          ;# handle void value
+            element::update $element 0
+        } else {
+            element::update $element $value
+        }
+    }
+
+    proc modified {this monitored} {                                                                    ;# number of monitored cells
+        $($this,path) yaxis configure -hide [expr {$monitored == 0}]                               ;# hide if no elements to display
+        updateMessage $this
+    }
+
+    proc updateMessage {this} {
+        # viewer may have been immediately deleted after construction when a cell of an invalid type was dropped:
+        if {[catch {classof $this}]} return
+        if {[llength [blt2DViewer::cells $this]] == 0} {
+            centerMessage $widget::($this,path) [message $this] $(plotBackground) $global::viewerMessageColor
+        } else {
+            centerMessage $widget::($this,path) {}
+        }
+    }
+
+    virtual proc message {this}
+
+    proc initializationConfiguration {this} {
+        return [concat\
+            [list -ymaximum $composite::($this,-ymaximum) -ymaximumcell $composite::($this,-ymaximumcell)]\
+            [blt2DViewer::_initializationConfiguration $this]\
+        ]
     }
 
 }
@@ -97,11 +146,7 @@ class dataBarChart {
         proc set-deletecommand {this value} {}                                                   ;# data is stored at switched level
 
         proc update {this y} {                                                      ;# y is either a valid number or the ? character
-            if {[string equal $y ?]} {                                                                          ;# handle void value
-                $($this,path) element configure $this -ydata 0
-            } else {
-                $($this,path) element configure $this -ydata $y
-            }
+            $($this,path) element configure $this -ydata $y
         }
 
     }
@@ -132,6 +177,10 @@ class dataSideBarChart {
         }
     }
 
+    proc message {this} {
+        return "side bar chart:\ndrop data cell(s)"
+    }
+
 }
 
 class dataStackedBarChart {
@@ -155,6 +204,18 @@ class dataStackedBarChart {
         }
     }
 
+    proc updateElement {this element seconds value} {                           ;# value is either a valid number or the ? character
+        if {[string equal $value ?]} {                                                                          ;# handle void value
+            dataBarChart::element::update $element 0
+        } else {
+            dataBarChart::element::update $element [expr {abs($value)}]                             ;# negative values make no sense
+        }
+    }
+
+    proc message {this} {
+        return "stacked bar chart:\ndrop data cell(s)"
+    }
+
 }
 
 class dataOverlapBarChart {
@@ -181,4 +242,25 @@ class dataOverlapBarChart {
         }
     }
 
+    proc updateElement {this element seconds value} {                           ;# value is either a valid number or the ? character
+        dataBarChart::_updateElement $this $element $seconds $value                                               ;# update as usual
+        set path $dataBarChart::($this,path)
+        set list {}           ;# now update elements display so that none hides the others (ordered with largest values in the back)
+        foreach name [$path element show] {
+            set value [$path element cget $name -ydata]
+            if {![string is double -strict $value]} {set value 0}                                                ;# may be undefined
+            lappend list [list $name $value]
+        }
+        set names {}
+        foreach list [lsort -real -index end -decreasing $list] {
+            foreach {name value} $list {}
+            lappend names $name
+        }
+        $path element show $names
+    }
+
+    proc message {this} {
+        return "overlap bar chart:\ndrop data cell(s)"
+    }
+
 }
