@@ -1,7 +1,7 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: font.tcl,v 2.7 2000/12/31 23:14:19 jfontain Exp $}
+# $Id: font.tcl,v 2.16 2004/01/01 11:39:06 jfontain Exp $
 
 
 # Note: on UNIX, for best results, make sure your X Window server has the proper dots-per-inch setting for your screen
@@ -16,7 +16,12 @@ class font {
     font configure $(mediumNormal) -weight normal                                                          ;# force weight to normal
     set (mediumItalic) [eval font create [font actual $(mediumNormal)]]
     font configure $(mediumItalic) -slant italic                                                            ;# force slant to italic
+    set (mediumBoldItalic) [eval font create [font actual $(mediumBold)]]
+    font configure $(mediumBoldItalic) -slant italic                                                        ;# force slant to italic
     set (smallNormal) [eval font create [font actual $(mediumNormal)]]
-    font configure $(smallNormal) -size [expr {round(0.8*[font actual $(mediumNormal) -size])}]  ;# small size is 80% of medium size
+    # small size is 80% of medium size:
+    font configure $(smallNormal) -size [expr {round(0.8 * [font actual $(mediumNormal) -size])}]
+    set (tinyNormal) [font create -family [font actual $(mediumNormal) -family] -size -10]
+    set (fixedNormal) [font create -family courier -weight normal -size [font actual $(mediumNormal) -size]]
 }
 
