@@ -1,13 +1,13 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: module.tcl,v 2.22 2001/02/28 21:00:47 jfontain Exp $}
+# $Id: module.tcl,v 2.49 2004/01/01 11:39:06 jfontain Exp $
 
 
 class module {
 
     proc module {this name index} {       ;# index may be forced (when coming from a save file), left empty for automatic generation
-        if {[string length $index]==0} {
+        if {[string length $index] == 0} {
             set index [newIndex $name]
         } else {                                                                                    ;# index was passed as parameter
             addIndex $name $index
@@ -26,7 +26,7 @@ class module {
                     perl::interp delete $($this,interpreter)
                 }
                 python {
-#                   python::interp delete $($this,interpreter)                        ### when multiple interpreters can be used ###
+                    python::interp delete $($this,interpreter)
                 }
                 default {
                     interp delete $($this,interpreter)
@@ -39,7 +39,10 @@ class module {
         deleteIndex $this
     }
 
-    proc newIndex {name} {              ;# find a new index for the module, by eventually finding a hole in that module indices list
+    # find a new index for the module, by eventually finding a hole in that module indices list.
+    # important: giving the index of a module that was just deleted to a new module (of the same name of course) is mandatory for
+    # module reloading to work properly
+    proc newIndex {name} {
         variable indices
 
         if {![info exists indices($name)]} {
@@ -47,7 +50,7 @@ class module {
         }
         set new 0
         foreach index $indices($name) {
-            if {$index!=$new} break                                                                                  ;# found a hole
+            if {$index != $new} break                                                                                ;# found a hole
             incr new
         }
         set indices($name) [lsort -integer [concat $indices($name) $new]]
@@ -61,7 +64,7 @@ class module {
             set indices($name) $index
             return
         }
-        if {[lsearch -exact $indices($name) $index]>=0} {
+        if {[lsearch -exact $indices($name) $index] >= 0} {
             error "trying to add an existing index: $index"
         }
         set indices($name) [lsort -integer [concat $indices($name) $index]]
@@ -72,7 +75,7 @@ class module {
 
         set name $($this,name)
         ldelete indices($name) $($this,index)
-        if {[llength $indices($name)]==0} {
+        if {[llength $indices($name)] == 0} {
             unset indices($name)
         }
     }
@@ -80,22 +83,36 @@ class module {
     proc load {this} {              ;# load a module in its own interpreter, in order to allow multiple instances of the same module
         set name $($this,name)
         set index $($this,index)
-        if {$index==0} {                                                                               ;# first instance of a module
+        if {$index == 0} {                                                                             ;# first instance of a module
             set namespace $name                                                                          ;# use original module name
         } else {
-            set namespace ${name}<$index>                                          ;# this is another instance of an existing module
+            set namespace ${name}<$index>                        ;# this is another instance of an existing module or a forced index
         }
 
         set directory [pwd]
-        cd $::packageDirectory($name)                                        ;# switch to module directory only during loading phase
+        cd $::package(directory,$name)                                       ;# switch to module directory only during loading phase
 
         set interpreter [interp create]                                                ;# use a separate interpreter for each module
+if {$global::withGUI} {
         $interpreter eval {                    ;# since Tk is not loaded in module interpreter, provide a background error procedure
             proc bgerror {error} {
                 puts stderr $::errorInfo
                 exit 1
             }
         }
+} else {                                       ;# for asynchronous modules, handle background errors exactly as for synchronous ones
+        $interpreter alias writeLog writeLog
+        interp eval $interpreter "namespace eval global {set debug $::global::debug}"
+        $interpreter eval {
+            proc bgerror {message} {
+                if {$::global::debug} {
+                    writeLog $::errorInfo critical
+                } else {
+                    writeLog $message critical
+                }
+            }
+        }
+}
         $interpreter eval "set auto_path [list $::auto_path]"                        ;# set packages paths list in child interpreter
         catch {$interpreter eval {package require {}}}    ;# then seek all packages locations (many pkgIndex.tcl files sourced here)
         # then intercept source command to be able to detect Perl modules:
@@ -103,7 +120,14 @@ class module {
         namespace eval ::$namespace {}                                                                    ;# create module namespace
         set ($this,namespace) $namespace                                                         ;# needed when loading Perl modules
         set ($this,terminate) 0                                            ;# needed in case loading fails and destructor is invoked
-        $interpreter eval "package require $name"                                                           ;# module is loaded here
+        # initialize counter copy to a minimal value (in case it is not done in the module body):
+        set ::${namespace}::data(updates) $global::integerMinimum
+        set ::${namespace}::data(identifier) $namespace              ;# initialize identifier which may be overridden in module code
+        if {[info exists ::package(exact,$name)]} {
+           $interpreter eval "package require -exact $name $::package(version,$name)"                       ;# module is loaded here
+        } else {
+           $interpreter eval "package require $name"                                                        ;# module is loaded here
+        }
         # we never get here if there is an error when the module is loaded
         switch $($this,type) {
             perl - python {
@@ -127,7 +151,20 @@ class module {
         set name $($this,name)
         set interpreter $($this,interpreter)
         set namespace $($this,namespace)
+if {$global::withGUI && ![string equal $name trace]} {                ;# no need to display update message for internal trace module
+        # module update procedure is only invoked if the module is synchronous, update message always being popped when updated:
+        namespace eval ::$namespace [subst -nocommands {
+            proc update {args} {
+                variable data
+
+                lifoLabel::push $global::messenger "\$data(identifier) data update..."
+                $interpreter eval ::${name}::update \$args
+                lifoLabel::pop $global::messenger
+            }
+        }]
+} else {
         namespace eval ::$namespace [subst -nocommands {proc update {args} {$interpreter eval ::${name}::update \$args}}]
+}
         set ($this,initialize) [procedureExists $this initialize]
         if {$($this,initialize)} {                                                                    ;# initialize procedure exists
             namespace eval ::$namespace [subst -nocommands {        ;# create an interface initialize procedure within new namespace
@@ -147,6 +184,7 @@ class module {
         set ($this,version) [$interpreter eval "package provide $name"]
         synchronize $this                                                ;# initialize namespace data from module in its interpreter
         validateColumnTitles $this
+        $interpreter alias exit exit                                       ;# redirect to parent interpreter (useful in daemon mode)
         # keep on eye on special module data array member "update"
         $interpreter alias _updated ::module::updated $this
         $interpreter eval "trace variable ::${name}::data(updates) w _updated"
@@ -155,16 +193,35 @@ class module {
         $interpreter alias popMessage ::modules::popMessage
         $interpreter alias flashMessage ::modules::flashMessage $name $namespace
         $interpreter alias traceMessage ::modules::trace $name $namespace
+if {$global::withGUI} {
+        if {[string equal $name instance]} {
+            $interpreter alias cellHistory ::databaseInstances::cellHistory
+            proc ::${namespace}::mapping {row column} "return \[$interpreter eval ${name}::mapping \$row \$column\]"
+        }
+}
+
     }
 
     proc updated {this args} {                             ;# Tcl module data was just updated. ignore already known trace arguments
         set namespace $($this,namespace)
-        lifoLabel::push $global::messenger "$namespace data update..."
-        update idletasks
+        set updates [$($this,interpreter) eval "set ::$($this,name)::data(updates)"]                     ;# retrieve updates counter
+        if {$updates <= [set ::${namespace}::data(updates)]} return                        ;# do nothing if counter did not increase
+        set asynchronous [expr {[lindex [set ::${namespace}::data(pollTimes)] 0] <= 0}]
+if {$global::withGUI} {
+        set trace [string equal $($this,name) trace]
+        if {$asynchronous && !$trace} {                               ;# no need to display update message for internal trace module
+            # module update procedure is never invoked so push update message here:
+            lifoLabel::push $global::messenger "[set ::${namespace}::data(identifier)] data update..."
+        }
+}
         array unset ::${namespace}::data {[0-9]*,[0-9]*}                         ;# first clear dynamic data (so rows can disappear)
         synchronize $this {[0-9]*,[0-9]*}                                            ;# just copy all dynamic data from module array
-        set ::${namespace}::data(updates) [$($this,interpreter) eval "set ::$($this,name)::data(updates)"]   ;# copy updates counter
-        lifoLabel::pop $global::messenger
+        set ::${namespace}::data(updates) $updates                                                           ;# copy updates counter
+if {$global::withGUI} {
+        if {$asynchronous && !$trace} {
+            lifoLabel::pop $global::messenger
+        }
+}
     }
 
     proc clear {this} {                                                     ;# act as if Tcl module code was updated with empty data
@@ -184,7 +241,7 @@ class module {
             }
             python {
                 # invoked only once in instance initialization, so copying form data only is enough
-                array set ::${namespace}::data [$interpreter eval "print formstring($name.form)"]
+                array set ::${namespace}::data [$interpreter eval formstring($name.form)]
             }
             default {
                 array set ::${namespace}::data [$interpreter eval "array get ::${name}::data {$pattern}"]
@@ -194,7 +251,7 @@ class module {
 
     proc validateColumnTitles {this} {
         foreach {name label} [array get ::$($this,namespace)::data *,label] {
-            if {[string first ? $label]>=0} {
+            if {[string first ? $label] >= 0} {
                 scan $name %u column
                 puts stderr "in $($this,namespace) module, column $column label contains a ? character: \"$label\""
                 exit 1
@@ -210,8 +267,7 @@ class module {
 
     # module interpreter source command, to do special processing if the module code is written in Perl
     proc source {this interpreter file} {                                                                       ;# a Tcl interpreter
-        set extension [file extension $file]
-        switch $extension {
+        switch [file extension $file] {
             .pm {
                 set ($this,type) perl          ;# remember that this is a Perl module so that destructor knows in case of load error
                 loadPerl $this $file
@@ -243,15 +299,33 @@ class module {
         $interpreter eval $perl::utilityFunctions
         # initialize static part of data:
         array set ::${namespace}::data [$interpreter eval hash_string(%${name}::data)]
+if {$global::withGUI} {
         proc ::${namespace}::update {} "                                                                  ;# create update procedure
             variable data
-            lifoLabel::push $global::messenger {$namespace data update...}
+
+            lifoLabel::push $global::messenger \"\$data(identifier) data update...\"
             $interpreter eval ${name}::update()
-            array unset data {\[0-9\]*,\[0-9\]*}                                 ;# first clear dynamic data (so rows can disappear)
-            array set data \[$interpreter eval array_string(@${name}::data)\]                                   ;# copy dynamic data
-            set data(updates) \[$interpreter eval \\$${name}::data{updates}\]                                ;# copy updates counter
+            set updates \[$interpreter eval \\$${name}::data{updates}\]                                  ;# retrieve updates counter
+            if {\$updates > \$data(updates)} {                                             ;# do nothing if counter did not increase
+                array unset data {\[0-9\]*,\[0-9\]*}                             ;# first clear dynamic data (so rows can disappear)
+                array set data \[$interpreter eval array_string(@${name}::data)\]                               ;# copy dynamic data
+                set data(updates) \$updates                                                                  ;# copy updates counter
+            }
             lifoLabel::pop $global::messenger
         "
+} else {                                                                                                 ;# do not use GUI messenger
+        proc ::${namespace}::update {} "
+            variable data
+
+            $interpreter eval ${name}::update()
+            set updates \[$interpreter eval \\$${name}::data{updates}\]
+            if {\$updates > \$data(updates)} {
+                array unset data {\[0-9\]*,\[0-9\]*}
+                array set data \[$interpreter eval array_string(@${name}::data)\]
+                set data(updates) \$updates
+            }
+        "
+}
         set ($this,initialize) [$interpreter eval int(defined(&${name}::initialize))]                      ;# Tcl compatible boolean
         if {[info exists ${namespace}::data(switches)]} {
             array set argumentRequired [set ${namespace}::data(switches)]
@@ -271,7 +345,7 @@ class module {
                         regsub -all {\\\\} \$value {\\\\\\\\} value                                           ;# escape back-slashes
                         regsub -all ' \$value {\\\\\\'} value                                         ;# and simple quote separators
                     }
-                    if {\[string length \$argument\]>0} {append argument ,}
+                    if {\[string length \$argument\] > 0} {append argument ,}
                     append argument '\$name','\$value'            ;# built a list used to initialize a hash in initialize subroutine
                 }
                 $interpreter eval ${name}::initialize(\$argument)
@@ -287,29 +361,49 @@ class module {
     proc loadPython {this file} {
         set name $($this,name)                                                                          ;# also the Perl module name
         set namespace $($this,namespace)
-        if {[catch {package require tclpython 1.1} message]} {                                                ;# load Python library
+        if {[catch {package require tclpython 3} message]} {                        ;# load Python library (see also in modules.tcl)
             error "$message\nis the tclpython package installed?"
         }
         set interpreter [python::interp new]                                                                               ;# create
         set ($this,interpreter) $interpreter                                               ;# the Python interpreter for this module
 
-        $interpreter eval "import sys\nsys.path.insert(0, '.')"             ;# so that module can be imported from current directory
-        $interpreter eval {from types import FunctionType}                       ;# to be able to find out whether a function exists
-        $interpreter eval {import re}                                    ;# regular expressions are required by internal Python code
-        $interpreter eval "import $name"                                                             ;# and import the Python module
-        $interpreter eval $python::utilityFunctions
+        $interpreter exec "import sys\nsys.path.insert(0, '.')"             ;# so that module can be imported from current directory
+        $interpreter exec {from types import FunctionType}                       ;# to be able to find out whether a function exists
+        $interpreter exec {import re}                                    ;# regular expressions are required by internal Python code
+        $interpreter exec "import $name"                                                             ;# and import the Python module
+        $interpreter exec $python::utilityFunctions
+        $interpreter exec "import signal\nsignal.signal(2, signal.SIG_DFL)"      ;# allow keyboard interrupts to stop moodss (as ^C)
         # initialize static part of data:
-        array set ::${namespace}::data [$interpreter eval "print formstring($name.form)"]
+        array set ::${namespace}::data [$interpreter eval formstring($name.form)]
+if {$global::withGUI} {
         proc ::${namespace}::update {} "                                                                  ;# create update procedure
             variable data
-            lifoLabel::push $global::messenger {$namespace data update...}
-            $interpreter eval $name.update()
-            array unset data {\[0-9\]*,\[0-9\]*}                                 ;# first clear dynamic data (so rows can disappear)
-            array set data \[$interpreter eval {print datastring($name.data)}\]                                 ;# copy dynamic data
-            set data(updates) \[$interpreter eval {print $name.form\['updates'\]}]                           ;# copy updates counter
+
+            lifoLabel::push $global::messenger \"\$data(identifier) data update...\"
+            $interpreter exec $name.update()
+            set updates \[$interpreter eval {$name.form\['updates'\]}]                                   ;# retrieve updates counter
+            if {\$updates > \$data(updates)} {                                             ;# do nothing if counter did not increase
+                array unset data {\[0-9\]*,\[0-9\]*}                             ;# first clear dynamic data (so rows can disappear)
+                array set data \[$interpreter eval datastring($name.data)\]                                     ;# copy dynamic data
+                set data(updates) \$updates                                                                  ;# copy updates counter
+            }
             lifoLabel::pop $global::messenger
         "
-        set ($this,initialize) [$interpreter eval "try: print type($name.initialize) == FunctionType\nexcept: print 0"]
+} else {                                                                                                 ;# do not use GUI messenger
+        proc ::${namespace}::update {} "
+            variable data
+
+            $interpreter exec $name.update()
+            set updates \[$interpreter eval {$name.form\['updates'\]}]
+            if {\$updates > \$data(updates)} {
+                array unset data {\[0-9\]*,\[0-9\]*}
+                array set data \[$interpreter eval datastring($name.data)\]
+                set data(updates) \$updates
+            }
+        "
+}
+        $interpreter exec "try: result = (type($name.initialize) == FunctionType)\nexcept: result = 0"
+        set ($this,initialize) [$interpreter eval result]
         if {[info exists ${namespace}::data(switches)]} {
             array set argumentRequired [set ${namespace}::data(switches)]
         }
@@ -328,17 +422,18 @@ class module {
                         regsub -all {\\\\} \$value {\\\\\\\\} value                                           ;# escape back-slashes
                         regsub -all ' \$value {\\\\\\'} value                                         ;# and simple quote separators
                     }
-                    if {\[string length \$argument\]>0} {append argument ,}
+                    if {\[string length \$argument\] > 0} {append argument ,}
                     append argument '\$name':'\$value'      ;# built a list used to initialize a dictionary in initialize subroutine
                 }
-                $interpreter eval $name.initialize({\$argument})
+                $interpreter exec $name.initialize({\$argument})
             "
         }
-        set ($this,terminate) [$interpreter eval "try: print type($name.terminate) == FunctionType\nexcept: print 0"]
+        $interpreter exec "try: result = (type($name.terminate) == FunctionType)\nexcept: result = 0"
+        set ($this,terminate) [$interpreter eval result]
         if {$($this,terminate)} {                                                                      ;# create terminate procedure
-            proc ::${namespace}::terminate {} "$interpreter eval $name.terminate()"
+            proc ::${namespace}::terminate {} "$interpreter exec $name.terminate()"
         }
-        set ($this,version) [$interpreter eval "print $name.__version__"]
+        set ($this,version) [$interpreter eval $name.__version__]
     }
 
 }
