@@ -1,11 +1,10 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: mounts.tcl,v 2.10 2001/01/27 15:08:19 jfontain Exp $}
+# $Id: mounts.tcl,v 2.21 2004/01/01 11:39:06 jfontain Exp $
 
 
-package provide mounts [lindex {$Revision: 2.10 $} 1]
-package require filesystem 1
+package provide mounts [lindex {$Revision: 2.21 $} 1]
 package require network 1
 
 namespace eval mounts {
@@ -27,34 +26,53 @@ namespace eval mounts {
     close $file
 
     proc initialize {optionsName} {
-        upvar $optionsName options
+        upvar 1 $optionsName options
         variable remote
         variable data
+        variable system
 
-        if {![catch {set locator $options(--remote)}]||![catch {set locator $options(-r)}]} {                   ;# remote monitoring
+        if {![catch {set locator $options(--remote)}] || ![catch {set locator $options(-r)}]} {                 ;# remote monitoring
             set data(pollTimes) {30 10 20 60 120 300 600}                                ;# poll less often when remotely monitoring
             foreach {remote(protocol) remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
             network::checkRemoteOutputEmptiness $remote(protocol) $remote(user) $remote(host)
             set data(identifier) mounts($remote(host))
-            set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) cat /proc/version"]
+            if {[string equal $::tcl_platform(platform) unix]} {
+                set remote(command) "$remote(protocol) -n -l $remote(user) $remote(host) uname"
+            } else {                                                                                                      ;# windows
+                set remote(command) "plink -batch $remote(host) uname"                             ;# host is rather a putty session
+            }
+            set file [open "| $remote(command)"]
             fileevent $file readable {set ::mounts::remote(busy) 0}
             vwait ::mounts::remote(busy)                                                               ;# do not hang user interface
+            set system [string trim [gets $file]]
         } else {
+            package require filesystem 1                                        ;# only needed locally (also unavailable on windows)
             set data(pollTimes) {20 5 10 30 60 120 300 600}
-            set file [open /proc/version]
+            set system $::tcl_platform(os)
+        }
+        switch $system {
+            Linux - SunOS {}
+            default {error "unsupported operating system: $system"}
         }
-        regexp {^[\d\.]+} [lindex [gets $file] 2] version                          ;# ignore extra characters, such as in 2.2.0-pre1
         if {[info exists remote]} {
             # detect errors early (but avoid write on pipe with no readers errors by reading whole data)
-            if {[catch {read $file} message]||[catch {close $file} message]} {
+            if {[catch {read $file} message] || [catch {close $file} message]} {
                 error "on remote host $remote(host) as user $remote(user): $message"
             }
-        } else {
-            close $file
-        }
-        if {[package vcompare $version 2]<0} {                                                               ;# check kernel version
-            puts stderr {mounts module needs at least a version 2 kernel}
-            exit 1
+            # now set command for updates:
+            switch $system {
+                Linux {
+                    set command {df -PT}                                                ;# use POSIX formatting to avoid line breaks
+                }
+                SunOS {
+                    set command {df -k && df -n}                                                ;# also retrieve the filesystem type
+                }
+            }
+            if {[string equal $::tcl_platform(platform) unix]} {
+                set remote(command) "$remote(protocol) -n -l $remote(user) $remote(host) $command"
+            } else {                                                                                                      ;# windows
+                set remote(command) "plink -batch $remote(host) $command"
+            }
         }
     }
 
@@ -74,15 +92,24 @@ namespace eval mounts {
 
     proc localUpdate {} {
         variable data
+        variable system
 
-        set file [open /etc/mtab]                        ;# must be opened each poll otherwise unmounted filesystems never disappear
-        while {[gets $file line]>=0} {
+        switch $system {
+            Linux {set name /etc/mtab}
+            SunOS {set name /etc/mnttab}
+        }
+        set file [open $name]                            ;# must be opened each poll otherwise unmounted filesystems never disappear
+        while {[gets $file line] >= 0} {
             foreach {device directory type flags dump order} $line {}
-            foreach {size total available used} [filesystem::statistics $directory] {}
-            if {$available==0} continue                              ;# ignore special cases, such as proc, pts, automount, usb, ...
-            set ratio [expr {$size/1024.0}]                    ;# take filesystem block size into account to get values in kilobytes
-            updateEntryData $directory $device $type [expr {round($total*$ratio)}] [expr {round($used*$ratio)}]\
-                [expr {round($available*$ratio)}] [expr {round(($used*100.0)/($used+$available))}]                      ;# used in %
+            foreach {size total available used blockSize} [filesystem::statistics $directory] {}
+            if {$available == 0} continue                            ;# ignore special cases, such as proc, pts, automount, usb, ...
+            # take filesystem block size into account to get values in kilobytes
+            switch $system {
+                Linux {set ratio [expr {$size / 1024.0}]}
+                SunOS {set ratio [expr {$blockSize / 1024.0}]}
+            }
+            updateEntryData $directory $device $type [expr {round($total * $ratio)}] [expr {round($used * $ratio)}]\
+                [expr {round($available * $ratio)}] [expr {round(($used * 100.0) / ($used + $available))}]              ;# used in %
             set current($directory) {}
         }
         close $file
@@ -94,26 +121,40 @@ namespace eval mounts {
         variable remote
 
         set remote(busy) 1
-        # use POSIX formatting to avoid line breaks
-        set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) /bin/df -PT"]
+        set file [open "| $remote(command)"]
         fileevent $file readable "mounts::remoteUpdated $file"               ;# do not hang user interface and other modules updates
     }
 
     proc remoteUpdated {file} {
         variable remote
         variable data
+        variable system
 
         gets $file                                                                                               ;# skip header line
-        while {[gets $file line]>=0} {
+        while {[gets $file line] >= 0} {
             lappend lines $line
         }
         read $file                                           ;# avoid write on pipe with no readers errors by reading remaining data
         if {[catch {close $file} message]} {
-            flashMessage "mounts error: $message"
+            flashMessage "error: $message"
             set lines {}                                                                       ;# consider data corrupted as a whole
         }
         foreach line $lines {
-            foreach {device type size used available capacity directory} $line {}
+            switch $system {
+                Linux {
+                    foreach {device type size used available capacity directory} $line {}
+                }
+                SunOS {
+                    if {[string equal [lindex $line 1] :]} {                                     ;# type line (example: /home : ufs)
+                        foreach {directory : type} $line {}
+                        foreach {device size used available capacity directory} $list($directory) {}
+                    } else {                       ;# statistics line (example: /dev/dsk/c0t0d0s7 13048715 817838 12100390 7% /home)
+                        set list([lindex $line end]) $line
+                        continue                                                                        ;# wait till we get the type
+                    }
+                }
+            }
+            if {$size == 0} continue                                                                     ;# skip unavailable entries
             updateEntryData $directory $device $type $size $used $available [string trimright $capacity %]
             set current($directory) {}
         }
@@ -141,7 +182,7 @@ namespace eval mounts {
     }
 
     proc cleanupEntriesData {currentDirectoriesName} {
-        upvar $currentDirectoriesName current
+        upvar 1 $currentDirectoriesName current
         variable index
         variable data
 
