@@ -1,12 +1,12 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: print.tcl,v 2.28 2000/12/31 23:14:19 jfontain Exp $}
+# $Id: print.tcl,v 2.42 2004/01/01 11:39:06 jfontain Exp $
 
 
 namespace eval print {
 
-    variable dotsPerMillimeter [expr {72/25.4}]                                          ;# 72 dots/inch is the default for printers
+    variable dotsPerMillimeter [expr {72 / 25.4}]                                        ;# 72 dots/inch is the default for printers
     variable helpMessage {Print the window to a printer device or to a file, in Postscript.}
     variable previewerWindow .grabber.printPreviewer
 
@@ -42,7 +42,7 @@ namespace eval print {
         radiobutton $frame.toCommand -variable print::printToFile -value 0
         grid $frame.toCommand -row $row -column 0 -sticky w
 
-        if {[string first %P $printCommand]<0} {                                       ;# command does not include printer parameter
+        if {[string first %P $printCommand] < 0} {                                     ;# command does not include printer parameter
             $frame.toCommand configure -text {with Command:}
             entry $frame.command -textvariable print::printCommand
             grid $frame.command -row $row -column 1 -sticky ew
@@ -58,7 +58,7 @@ namespace eval print {
             set entry [new comboEntry $frame -font $widget::option(entry,font) -list $entries]
             lappend objects $entry
             composite::configure $entry entry -textvariable print::printer
-            if {[llength $entries]<=3} {
+            if {[llength $entries] <= 3} {
                 composite::configure $entry button -listheight [llength $entries]
             }
             composite::configure $entry button scroll -selectmode single
@@ -69,7 +69,7 @@ namespace eval print {
         if {[catch {exec gs --version} version]} {
             $button configure -state disabled
             lappend objects [new widgetTip -path $button -text {could not get gs version}]
-        } elseif {[package vcompare $version 5.20]<0} {                                     ;# -dWindowID switch did not work before
+        } elseif {[package vcompare $version 5.20] < 0} {                                   ;# -dWindowID switch did not work before
             $button configure -state disabled
             lappend objects [new widgetTip -path $button -text {requires gs version above 5.20}]
         }
@@ -114,6 +114,8 @@ namespace eval print {
         composite::configure $entry button scroll -selectmode single
         grid $widget::($entry,path) -row $row -column 1 -columnspan 2 -sticky ew
 
+        grid columnconfigure $frame 1 -weight 1
+
         dialogBox::display $dialog $frame
         # make sure dialog box does not appear in printout
         widget::configure $dialog -command "delete $dialog; print::updateGlobals; update; print::print"
@@ -148,10 +150,10 @@ namespace eval print {
 
     proc inquireFileToPrintTo {parentPath} {
         set file [tk_getSaveFile\
-            -title {moodss: Print to file} -parent $parentPath -initialdir [pwd] -defaultextension .ps\
-            -filetypes {{Postscript .ps} {{All files} *}} -initialfile $print::fileToPrintTo\
+            -title {moodss: Print to file} -parent $parentPath -initialdir [file dirname $print::fileToPrintTo]\
+            -defaultextension .ps -filetypes {{Postscript .ps} {{All files} *}} -initialfile $print::fileToPrintTo\
         ]
-        if {[string length $file]>0} {                                                                              ;# not cancelled
+        if {[string length $file] > 0} {                                                                             ;# not canceled
             set print::fileToPrintTo $file
         }
     }
@@ -172,44 +174,60 @@ namespace eval print {
         set global::printPaperSize $printPaperSize
     }
 
+    proc canvasPrintArea {} {                                                                  ;# calculate visible area coordinates
+        set canvas $global::canvas
+        # first calculate visible area coordinates
+        foreach {left top right bottom} [$canvas cget -scrollregion] {}
+        set width [expr {$right - $left}]
+        set height [expr {$bottom - $top}]
+        foreach {minimum maximum} [$canvas xview] {}
+        set left [expr {$left + ($minimum * $width)}]
+        foreach {minimum maximum} [$canvas yview] {}
+        set top [expr {$top + ($minimum * $height)}]
+        scan [winfo geometry $canvas] %ux%u width height                                                ;# now get visible area size
+        set right [expr {$left + $width}]
+        set bottom [expr {$top + $height}]
+        set items {}
+        foreach item [$canvas find all] {
+            set print 1
+            foreach tag [$canvas gettags $item] {
+                # do not take icons into account (they are only printed if visible among the printable items):
+                if {[string match icon(*) $tag]} {
+                    set print 0
+                    break
+                }
+            }
+            if {$print} {
+                lappend items $item
+            }
+        }
+        if {[llength $items] > 0} {                                  ;# there may be no items in the canvas if no modules are loaded
+            foreach {boundsLeft boundsTop boundsRight boundsBottom} [eval $canvas bbox $items] {}
+            if {$boundsLeft > $left} {set left $boundsLeft}
+            if {$boundsRight < $right} {set right $boundsRight}
+            if {$boundsTop > $top} {set top $boundsTop}
+            if {$boundsBottom < $bottom} {set bottom $boundsBottom}
+        }
+        return [list $left $top [expr {$right - $left}] [expr {$bottom - $top}]]
+    }
+
     proc postscriptOptions {{gsOutput 0} {pageWidthName {}} {pageHeightName {}}} {
         variable printOrientation
         variable printPaperSize
         variable printPalette
         variable dotsPerMillimeter
-        if {[string length $pageWidthName]>0} {
-            upvar $pageWidthName pageWidth
+        if {[string length $pageWidthName] > 0} {
+            upvar 1 $pageWidthName pageWidth
         }
-        if {[string length $pageHeightName]>0} {
-            upvar $pageHeightName pageHeight
+        if {[string length $pageHeightName] > 0} {
+            upvar 1 $pageHeightName pageHeight
         }
 
         update                                          ;# make sure everything is redrawn properly in case dialog box hid something
+        foreach {left top width height} [canvasPrintArea] {}
 
-        set canvas $global::canvas
-        # first calculate visible area coordinates
-        foreach {left top right bottom} [$canvas cget -scrollregion] {}
-        set width [expr {$right-$left}]
-        set height [expr {$bottom-$top}]
-        foreach {minimum maximum} [$canvas xview] {}
-        set left [expr {$left+($minimum*$width)}]
-        foreach {minimum maximum} [$canvas yview] {}
-        set top [expr {$top+($minimum*$height)}]
-        scan [winfo geometry $canvas] %ux%u width height                                                ;# now get visible area size
-        set right [expr {$left+$width}]
-        set bottom [expr {$top+$height}]
-        set box [$canvas bbox all]                                                      ;# then eventually minimize to visible items
-        if {[llength $box]>0} {                                      ;# there may be no items in the canvas if no modules are loaded
-            foreach {boundsLeft boundsTop boundsRight boundsBottom} [$canvas bbox all] {}
-            if {$boundsLeft>$left} {set left $boundsLeft}
-            if {$boundsRight<$right} {set right $boundsRight}
-            if {$boundsTop>$top} {set top $boundsTop}
-            if {$boundsBottom<$bottom} {set bottom $boundsBottom}
-        }
-        set width [expr {$right-$left}]
-        set height [expr {$bottom-$top}]
         set inch 25.4                                                                                                 ;# millimeters
-        set margin [expr {$inch/2}]                                                                 ;# all dimensions in millimeters
+        set margin [expr {$inch / 2}]                                                               ;# all dimensions in millimeters
         switch -glob $printPaperSize {
             A3* {
                 set pageWidth 297
@@ -220,16 +238,16 @@ namespace eval print {
                 set pageHeight 297
             }
             executive* {
-                set pageWidth [expr {7.5*$inch}]
-                set pageHeight [expr {10*$inch}]
+                set pageWidth [expr {7.5 * $inch}]
+                set pageHeight [expr {10 * $inch}]
             }
             legal* {
-                set pageWidth [expr {8.5*$inch}]
-                set pageHeight [expr {14*$inch}]
+                set pageWidth [expr {8.5 * $inch}]
+                set pageHeight [expr {14 * $inch}]
             }
             default {
-                set pageWidth [expr {8.5*$inch}]                                                                           ;# letter
-                set pageHeight [expr {11*$inch}]
+                set pageWidth [expr {8.5 * $inch}]                                                                         ;# letter
+                set pageHeight [expr {11 * $inch}]
             }
         }
         set pageX ${margin}m
@@ -237,7 +255,7 @@ namespace eval print {
         if {$rotate} {
             set pageY ${margin}m
         } else {
-            set pageY [expr {$pageHeight-$margin}]m
+            set pageY [expr {$pageHeight - $margin}]m
         }
         # now make sure everything fits in page
         if {$rotate} {                                                                                 ;# swap page width and height
@@ -246,17 +264,17 @@ namespace eval print {
             set pageHeight $value
             unset value
             if {$gsOutput} {                                   ;# for gs, the printout is not rotated, only the page size is changed
-                set pageY [expr {$pageHeight-$margin}]m
+                set pageY [expr {$pageHeight - $margin}]m
             }
         }
-        set printWidth [expr {($pageWidth-(2*$margin))*$dotsPerMillimeter}]                                             ;# in pixels
-        set printHeight [expr {($pageHeight-(2*$margin))*$dotsPerMillimeter}]
+        set printWidth [expr {($pageWidth - (2 * $margin)) * $dotsPerMillimeter}]                                       ;# in pixels
+        set printHeight [expr {($pageHeight - (2 * $margin)) * $dotsPerMillimeter}]
         set ratio 1                                                                  ;# use identical horizontal and vertical ratios
-        if {$printWidth<$width} {
-            set ratio [expr {$printWidth/$width}]
+        if {$printWidth < $width} {
+            set ratio [expr {$printWidth / $width}]
         }
-        if {($printHeight<$height)&&(($printHeight/$height)<$ratio)} {
-            set ratio [expr {$printHeight/$height}]
+        if {($printHeight < $height) && (($printHeight / $height) < $ratio)} {
+            set ratio [expr {$printHeight / $height}]
         }
         if {$gsOutput} {                                       ;# for gs, the printout is not rotated, only the page size is changed
             set rotate 0
@@ -265,8 +283,8 @@ namespace eval print {
             -colormode $printPalette -rotate $rotate -x $left -y $top -width $width -height $height\
             -pageanchor nw -pagex $pageX -pagey $pageY\
         ]
-        if {$ratio<1} {                                                                                     ;# size reduction needed
-            lappend options -pagewidth [expr {$ratio*$width}] -pageheight [expr {$ratio*$height}]
+        if {$ratio < 1} {                                                                                   ;# size reduction needed
+            lappend options -pagewidth [expr {$ratio * $width}] -pageheight [expr {$ratio * $height}]
         }
         return $options
     }
@@ -277,7 +295,7 @@ namespace eval print {
         variable printCommand
         variable printer
 
-        blt::busy hold .
+        busy 1
         set options [postscriptOptions]                       ;# do first as internal updating is done for proper geometry assertion
         if {$printToFile} {
             lifoLabel::push $global::messenger "printing to file $fileToPrintTo..."
@@ -288,22 +306,22 @@ namespace eval print {
         if {$printToFile} {
             lappend options -file $fileToPrintTo
             eval $global::canvas postscript $options
-        } else {                             ;# gather data first in case window gets obscured by printing utility (which may be gv)
+        } else {                             ;# gather data first in case window gets obscured by printing utility (which may be gs)
             set data [eval $global::canvas postscript $options]
-            if {[string first %P $printCommand]<0} {                                   ;# command does not include printer parameter
+            if {[string first %P $printCommand] < 0} {                                 ;# command does not include printer parameter
                 set command $printCommand                                                                               ;# use as is
             } else {                                                                                         ;# use selected printer
                 regsub -all %P $printCommand [scan $printer %s] command       ;# only use first word (format is "name (alias, ...)")
             }
-            if {[catch {set channel [open |$command w]} message]} {
-                tk_messageBox -title {moodss: Error when printing} -type ok -icon error -message $message
-            } else {
-                puts -nonewline $channel $data
-                close $channel
+            if {\
+                [catch {set channel [open |$command w]} message] ||\
+                [catch {puts -nonewline $channel $data} message] || [catch {close $channel} message]\
+            } {
+                tk_messageBox -title {moodss: Error when printing} -type ok -icon error -message "$command: $message"
             }
         }
         lifoLabel::pop $global::messenger
-        blt::busy forget .
+        busy 0
     }
 
     proc preview {} {
@@ -334,7 +352,7 @@ namespace eval print {
             pack $widget::($viewer,path)
         }
         lower $previewerWindow                                                                             ;# do not obstruct canvas
-        blt::busy hold .
+        busy 1
         lifoLabel::push $global::messenger "previewing with gs..."
         update idletasks                                                                             ;# make sure message is visible
         set options [postscriptOptions 1 width height]        ;# do first as internal updating is done for proper geometry assertion
@@ -345,7 +363,21 @@ namespace eval print {
         composite::configure $viewer -file $file -pagewidth $width -pageheight $height
         printViewer::refresh $viewer
         lifoLabel::pop $global::messenger
-        blt::busy forget .
+        busy 0
+    }
+
+    proc createTemporaryCanvasShot {} {
+        update idletasks                                                            ;# try to make sure everything is drawn properly
+        foreach {left top width height} [canvasPrintArea] {}
+        set file [temporaryFileName png]
+        set channel [open\
+            "|gs -q -dBATCH -dNOPROMPT -sDEVICE=png256 -g${width}x${height} -r$printViewer::(pixelsPerInch) -sOutputFile=$file -"\
+            w\
+        ]
+        $global::canvas postscript -colormode color -x 0 -y 0 -width $width -height $height -pageanchor sw -pagex 0 -pagey 0\
+            -channel $channel
+        close $channel
+        return $file
     }
 
 }
