@@ -1,14 +1,15 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: random.tcl,v 1.40 2001/01/27 17:42:00 jfontain Exp $}
+# $Id: random.tcl,v 1.53 2004/01/01 11:39:06 jfontain Exp $
 
 # module sample file with random data
 
 # corresponding pkgIndex.tcl file content:
-# package ifneeded random 4.0 "source [file join $dir random.tcl]"
+# package ifneeded random 1.45 [list source [file join $dir random.tcl]]
 
-package provide random [lindex {$Revision: 1.40 $} 1]
+# Note: see version number management requirements in moodss HTML documentation (Module Development section)
+package provide random [lindex {$Revision: 1.53 $} 1]
 
 namespace eval random {                                                                    ;# namespace name must match package name
     # Module data is arranged in a single array that can be used by many views, such as tables, graphs, pies,...
@@ -20,7 +21,7 @@ namespace eval random {
     #   changed in the initialize procedure).
     # Column numbers must be positive, starting from 0 included. Column members are:
     #   - label: the title displayed on the top cell of the data column.
-    #   - type: valid types are those that the lsort command can handle.
+    #   - type: valid types are those that the lsort command can handle, plus the clock type
     #   - message: help text that appears in a widget tip when the mouse cursor is left over the column title cell for a short
     #     while.
     #   - anchor: optional member. Column data is either centered by default (center), tucked to the left (left) or right side
@@ -51,9 +52,12 @@ namespace eval random {
     #   appropriate initialize procedure must be provided by the module (see initialize procedure example in this module).
     #   The core will take care of parsing the command line and reject any invalid switch / value combination for the module.
     #   The switches value may not be changed in the initialize procedure.
+    #   Note: if a --daemon special option exists (must take no arguments), it is automatically set when in daemon mode (moomps).
     # - identifier: optional, ignore for moodss versions before 8.0. A string that uniquely identifies this module. It will be
     #   displayed by the core in the initial data tables title area and data cell labels in viewers. This feature can be used for
     #   example in modules that gather data from a remote host: in such a case, the identifier could be set to dataType(hostName).
+    # - resizableColumns: optional. A boolean value (0 or 1) which specifies whether displayed data table(s) columns can be manually
+    #   resized by the user with the mouse. Not available on a per view basis.
     array set data {
         updates 0
         0,label name 0,type ascii 0,message {user name}
@@ -67,7 +71,7 @@ namespace eval random {
             {indices {0 1 3 4} sort {1 decreasing}}
             {indices {0 2 4} sort {2 decreasing}}
         }
-        switches {-a 0 --asynchronous 0 -i 0 --identify 0}
+        switches {-a 0 --asynchronous 0 -i 0 --identify 0 --daemon 0}
     }
     set file [open random.htm]                                               ;# module directory is current during the loading phase
     set data(helpText) [read $file]                                   ;# initialize HTML help data from file in module sub-directory
@@ -91,19 +95,19 @@ namespace eval random {
     # In all cases, data members other than updates and switches can be set or updated in the initialize procedure, and taken into
     # account by the core.
     proc initialize {optionsName} {
-        upvar $optionsName options
+        upvar 1 $optionsName options
         variable asynchronous
         variable data
 
-        set asynchronous [expr {[info exists options(-a)]||[info exists options(--asynchronous)]}]
+        set asynchronous [expr {[info exists options(-a)] || [info exists options(--asynchronous)]}]
         if {$asynchronous} {
             # for a module to be asynchronous, the pollTimes member must be a single negative integer value. It then represents the
             # preferred time interval for viewers that require one.
             array set data {pollTimes -10}
             after 3000 random::update                                                                             ;# boot simulation
         }
-        if {[info exists options(-i)]||[info exists options(--identify)]} {                   ;# generate a unique module identifier
-            set data(identifier) "random [expr {int(rand()*100)}]"
+        if {[info exists options(-i)] || [info exists options(--identify)]} {                 ;# generate a unique module identifier
+            set data(identifier) "random [expr {int(rand() * 100)}]"
         }
     }
 
@@ -127,17 +131,24 @@ namespace eval random {
         variable asynchronous
 
         array set data "
-            0,0 John\\nWo 0,1 [format %.1f [expr {rand()*30}]] 0,2 [expr {100+int(rand()*50)}] 0,3 [expr {10+int(rand()*50)}] 0,4 cc
-            1,0 Bill 1,1 [format %.1f [expr {rand()*3}]] 1,2 [expr {300+int(rand()*100)}] 1,3 [expr {30+int(rand()*100)}] 1,4 xedit
-            2,0 Anny\\nDoe 2,1 [format %.1f [expr {rand()*5}]] 2,2 [expr {200+int(rand()*30)}] 2,3 [expr {20+int(rand()*30)}] 2,4 ps
-            4,0 Peter\\nWard 4,1 [format %.1f [expr {rand()*8}]] 4,2 [expr {50+int(rand()*10)}] 4,3 [expr {5+int(rand()*10)}] 4,4 ls
-            6,0 Laura 6,1 [format %.1f [expr {rand()*20}]] 6,2 [expr {90+int(rand()*20)}] 6,3 [expr {9+int(rand()*20)}] 6,4 emacs
-            3,0 Robert 3,1 [format %.1f [expr {rand()*10}]] 3,2 [expr {500+int(rand()*150)}] 3,3 [expr {50+int(rand()*150)}] 3,4 top
-            9,0 Laura 9,1 [format %.1f [expr {rand()*30}]] 9,2 [expr {100+int(rand()*50)}] 9,3 [expr {10+int(rand()*50)}] 9,4 cc
+            0,0 John\\nWo 0,1 [format %.1f [expr {rand() * 30}]] 0,2 [expr {100 + int(rand() * 50)}]
+                0,3 [expr {10 + int(rand() * 50)}] 0,4 cc
+            1,0 Bill 1,1 [format %.1f [expr {rand() * 3}]] 1,2 [expr {300 + int(rand() * 100)}]
+                1,3 [expr {30 + int(rand() * 100)}] 1,4 xedit
+            2,0 Anny\\nDoe 2,1 [format %.1f [expr {rand() * 5}]] 2,2 [expr {200 + int(rand() * 30)}]
+                2,3 [expr {20 + int(rand() * 30)}] 2,4 ps
+            4,0 Peter\\nWard 4,1 [format %.1f [expr {rand() * 8}]] 4,2 [expr {50 + int(rand() * 10)}]
+                4,3 [expr {5 + int(rand() * 10)}] 4,4 ls
+            6,0 Laura 6,1 [format %.1f [expr {rand() * 20}]] 6,2 [expr {90 + int(rand() * 20)}]
+                6,3 [expr {9 + int(rand() * 20)}] 6,4 emacs
+            3,0 Robert 3,1 [format %.1f [expr {rand() * 10}]] 3,2 [expr {500 + int(rand() * 150)}]
+                3,3 [expr {50 + int(rand() * 150)}] 3,4 top
+            9,0 Laura 9,1 [format %.1f [expr {rand() * 30}]] 9,2 [expr {100 + int(rand() * 50)}]
+                9,3 [expr {10 + int(rand() * 50)}] 9,4 cc
         "
         incr data(updates)                         ;# write variable so that update can be detected with an eventual tcl write trace
         if {$asynchronous} {                                                           ;# eventually simulate asynchronous operation
-            after [expr {2000+round(rand()*8000)}] random::update
+            after [expr {2000 + round(rand() * 8000)}] random::update
         }
     }
 }
