@@ -1,10 +1,10 @@
-# copyright (C) 1997-2001 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: route.tcl,v 2.9 2001/01/27 15:08:19 jfontain Exp $}
+# $Id: route.tcl,v 2.17 2004/01/01 11:39:06 jfontain Exp $
 
 
-package provide route [lindex {$Revision: 2.9 $} 1]
+package provide route [lindex {$Revision: 2.17 $} 1]
 package require network 1
 
 namespace eval route {
@@ -27,23 +27,28 @@ namespace eval route {
     close $file
 
     proc initialize {optionsName} {
-        upvar $optionsName options
+        upvar 1 $optionsName options
         variable lookup
         variable remote
         variable data
         variable routeFile
 
-        set lookup [expr {![info exists options(-n)]&&![info exists options(--numeric)]}]            ;# host or network names lookup
-        if {![catch {set locator $options(--remote)}]||![catch {set locator $options(-r)}]} {                   ;# remote monitoring
+        set lookup [expr {![info exists options(-n)] && ![info exists options(--numeric)]}]          ;# host or network names lookup
+        if {![catch {set locator $options(--remote)}] || ![catch {set locator $options(-r)}]} {                 ;# remote monitoring
             set data(pollTimes) {20 10 30 60 120 300 600}                                ;# poll less often when remotely monitoring
             foreach {remote(protocol) remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
             network::checkRemoteOutputEmptiness $remote(protocol) $remote(user) $remote(host)
             set data(identifier) route($remote(host))
-            set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) cat /proc/net/route"]
+            if {[string equal $::tcl_platform(platform) unix]} {
+                set remote(command) "$remote(protocol) -n -l $remote(user) $remote(host) cat /proc/net/route"
+            } else {                                                                                                      ;# windows
+                set remote(command) "plink -batch $remote(host) cat /proc/net/route"               ;# host is rather a putty session
+            }
+            set file [open "| $remote(command)"]
             fileevent $file readable {set ::route::remote(busy) 0}
             vwait ::route::remote(busy)                                                                     ;# allow GUI interaction
             # detect errors early (but avoid write on pipe with no readers errors by reading whole data)
-            if {[catch {read $file} message]||[catch {close $file} message]} {
+            if {[catch {read $file} message] || [catch {close $file} message]} {
                 error "on remote host $remote(host) as user $remote(user): $message"
             }
         } else {
@@ -66,7 +71,7 @@ namespace eval route {
             if {![info exists routeFile]} {                                ;# start data gathering process in a non blocking fashion
                 if {$remote(busy)} return                                           ;# core invocation while waiting for remote data
                 set remote(busy) 1
-                set file [open "| /usr/bin/$remote(protocol) -n -l $remote(user) $remote(host) cat /proc/net/route"]
+                set file [open "| $remote(command)"]
                 # do not hang GUI, allow other modules updates
                 fileevent $file readable "set ::route::routeFile $file; ::route::update"
                 return                                                                                       ;# wait for remote data
@@ -75,13 +80,13 @@ namespace eval route {
             seek $routeFile 0                                                                       ;# rewind before retrieving data
         }
         gets $routeFile                                                                                          ;# skip header line
-        while {[gets $routeFile line]>=0} {
+        while {[gets $routeFile line] >= 0} {
             lappend lines $line
         }
         if {[info exists remote]} {                                                 ;# closing is necessary since seek does not work
             read $routeFile                                  ;# avoid write on pipe with no readers errors by reading remaining data
             if {[catch {close $routeFile} message]} {                                    ;# communication error can be detected here
-                flashMessage "route error: $message"
+                flashMessage "error: $message"
                 set lines {}                                                                   ;# consider data corrupted as a whole
             }
             unset routeFile
