@@ -2,7 +2,7 @@
 # the next line restarts using the interpreter \
 exec tclsh "$0" "$@"
 
-set rcsId {$Id: source.tcl,v 1.11 2001/01/25 20:56:18 jfontain Exp $}
+# $Id: source.tcl,v 1.17 2003/01/27 21:45:09 jfontain Exp $
 
 # This utility can be used to combine all sourced files making a Tcl
 # program into a single file, so that the source command is no longer
@@ -16,9 +16,9 @@ set rcsId {$Id: source.tcl,v 1.11 2001/01/25 20:56:18 jfontain Exp $}
 # type "source.tcl -h" for usage.
 
 proc sourceCode {fileName} {
-    global sourced directories
+    global sourced directories duplicates
 
-    if {[info exists sourced($fileName)]} {
+    if {!$duplicates&&[info exists sourced($fileName)]} {
         return {}                                                                     ;# do not process the same file more than once
     }
     set sourced($fileName) {}
@@ -39,15 +39,20 @@ proc sourceCode {fileName} {
             exit 1
         }
     }
-    set skip 0
+    # use ascii code for $ so that RCS does not mess up regular expression when checking in this file:
+    set strip [regexp -line {\044Id: [^\044]+ jfontain [^\044]+ \044} [read $file]]       ;# look for own RCS identification in file
+    seek $file 0
+    if {$strip} {set skip 0}
     while {[gets $file line]>=0} {
-        set continues [string match {*\\} $line]
-        if {$skip||[regexp {^[ 	]*#} $line]} {                                        ;# skip comments and eventually following line
-            set skip $continues
-            continue
-        }
-        if {!$continues} {
-            regsub {[ 	]*;#.*$} $line {} line                  ;# eventually remove trailing comments or blanks on terminated lines
+        if {$strip} {                             ;# since this is one of our own source files, it is safe to strip comments from it
+            set continues [string match {*\\} $line]
+            if {$skip||[regexp {^[ 	]*#} $line]} {                                    ;# skip comments and eventually following line
+                set skip $continues
+                continue
+            }
+            if {!$continues} {
+                regsub {[ 	]*;#.*$} $line {} line              ;# eventually remove trailing comments or blanks on terminated lines
+            }
         }
         if {[scan $line { source %s} name]>0} {
             append code [sourceCode [subst $name]]
@@ -66,11 +71,15 @@ if {[string match -h* [lindex $argv 0]]} {
     exit
 }
 
+set duplicates 0
 set output stdout
 set directories .                      ;# always start with current directory for files included through the source command facility
 
 foreach {option value} $argv {
     switch -- $option {
+        -d {
+            set duplicates $value
+        }
         -I {
             lappend directories $value
         }
